#include "Timer0.h"

CTimer0 Timer0;   // the CTimer0 instance

//-------------------------------------------------------------------
// The constructor provides all the required initialisation for
// this device.
//
CTimer0::CTimer0()
{
   TCNT0 = TIMER0_TIMEOUT;
   TCCR0 = 0x04;
   TIMSK |= SB(TOIE0);        // enable overflow interrupts
   mFlag = 0;
}

//-------------------------------------------------------------------
// Unlikely to be used in an embedded app.
//
CTimer0::~CTimer0()
{
   TIMSK &= ~SB(TOIE0);       // disable Timer0 timeout interrupt
}

//-------------------------------------------------------------------
// Probe to check if an interrupt has occurred. Should really use
// an atomic access that saves the processor interrupt state.
//
int CTimer0::GetFlag(void)
{
   int flag;

   cli();
   flag = mFlag;
   sei();
   return flag;
}

//-------------------------------------------------------------------
// Reset the interrupt flag. Should also be interrupt safe.
//
void CTimer0::ResetFlag(void)
{
   cli();
   mFlag = 0;
   sei();
}

//-------------------------------------------------------------------
// The overflow interrupt handler. Note it is only a friend of the
// device class, but has direct access to class data members. Note
// that it must know the name of the CTimer0 class instance.
//
void CTimer0Interrupt::OverflowInterrupt(void)
{
   TCNT0 = TIMER0_TIMEOUT;    // restart the timeout 
   Timer0.mFlag = 1;          // set the mFlag in CTimer0
}
